@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,20 +18,22 @@
 """
 sabnzbd.config - Configuration Support
 """
-import argparse
+
 import logging
 import os
 import re
 import shutil
 import threading
 import uuid
+import io
+import zipfile
 from typing import List, Dict, Any, Callable, Optional, Union, Tuple
 from urllib.parse import urlparse
 
 import configobj
 
-import sabnzbd.misc
-from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY
+import sabnzbd
+from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY, CONFIG_BACKUP_FILES, DEF_INI_FILE
 from sabnzbd.decorators import synchronized
 from sabnzbd.filesystem import clip_path, real_path, create_real_path, renamer, remove_file, is_writable
 
@@ -39,13 +41,11 @@ CONFIG_LOCK = threading.Lock()
 SAVE_CONFIG_LOCK = threading.Lock()
 
 
-CFG: configobj.ConfigObj  # Holds INI structure
+CFG_OBJ: configobj.ConfigObj  # Holds INI structure
 # during re-write this variable is global
 # to allow direct access to INI structure
 
-database = {}  # Holds the option dictionary
-
-modified = False  # Signals a change in option dictionary
+CFG_MODIFIED = False  # Signals a change in option dictionary
 # Should be reset after saving to settings file
 
 RE_PARAMFINDER = re.compile(r"""(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)""")
@@ -56,14 +56,13 @@ class Option:
 
     def __init__(self, section: str, keyword: str, default_val: Any = None, add: bool = True, protect: bool = False):
         """Basic option
-        `section`     : single section or comma-separated list of sections
-                        a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
-        `keyword`     : keyword in the (last) section
+        `section`     : single section for this option
+        `keyword`     : keyword in the section
         `default_val` : value returned when no value has been set
         `callback`    : procedure to call when value is successfully changed
         `protect`     : Do not allow setting via the API (specifically set_dict)
         """
-        self.__sections = section.split(",")
+        self.__section = section
         self.__keyword: str = keyword
         self.__default_val: Any = default_val
         self.__value: Any = None
@@ -72,13 +71,7 @@ class Option:
 
         # Add myself to the config dictionary
         if add:
-            global database
-            anchor = database
-            for section in self.__sections:
-                if section not in anchor:
-                    anchor[section] = {}
-                anchor = anchor[section]
-            anchor[keyword] = self
+            add_to_database(section, keyword, self)
 
     def get(self) -> Any:
         """Retrieve value field"""
@@ -104,14 +97,23 @@ class Option:
 
     def set(self, value: Any):
         """Set new value, no validation"""
-        global modified
+        global CFG_MODIFIED
         if value is not None:
             if isinstance(value, list) or isinstance(value, dict) or value != self.__value:
                 self.__value = value
-                modified = True
+                CFG_MODIFIED = True
                 if self.__callback:
                     self.__callback()
 
+    @property
+    def section(self) -> Any:
+        return self.__section
+
+    @property
+    def keyword(self) -> Any:
+        return self.__keyword
+
+    @property
     def default(self) -> Any:
         return self.__default_val
 
@@ -119,10 +121,6 @@ class Option:
         """Set callback function"""
         self.__callback = callback
 
-    def ident(self):
-        """Return section-list and keyword"""
-        return self.__sections, self.__keyword
-
 
 class OptionNumber(Option):
     """Numeric option class, int/float is determined from default value."""
@@ -153,7 +151,7 @@ class OptionNumber(Option):
                 else:
                     value = float(value)
             except ValueError:
-                value = super().default()
+                value = super().default
             if self.__validation:
                 _, val = self.__validation(value)
                 super().set(val)
@@ -192,7 +190,7 @@ class OptionDir(Option):
         section: str,
         keyword: str,
         default_val: str = "",
-        apply_umask: bool = False,
+        apply_permissions: bool = False,
         create: bool = True,
         validation: Optional[Callable] = None,
         writable: bool = True,
@@ -200,7 +198,7 @@ class OptionDir(Option):
     ):
         self.__validation: Optional[Callable] = validation
         self.__root: str = ""  # Base directory for relative paths
-        self.__apply_umask: bool = apply_umask
+        self.__apply_permissions: bool = apply_permissions
         self.__create: bool = create
         self.__writable: bool = writable
         super().__init__(section, keyword, default_val, add=add)
@@ -220,7 +218,9 @@ class OptionDir(Option):
         if value:
             path = real_path(self.__root, value)
             if self.__create and not os.path.exists(path):
-                _, path, _ = create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
+                _, path, _ = create_real_path(
+                    self.keyword, self.__root, value, self.__apply_permissions, self.__writable
+                )
         return path
 
     def get_clipped_path(self) -> str:
@@ -249,11 +249,11 @@ class OptionDir(Option):
         if value is not None and (value != self.get() or create):
             value = value.strip()
             if self.__validation:
-                error, value = self.__validation(self.__root, value, super().default())
+                error, value = self.__validation(self.__root, value, super().default)
             if not error:
                 if value and (self.__create or create):
                     res, path, error = create_real_path(
-                        self.ident()[1], self.__root, value, self.__apply_umask, self.__writable
+                        self.keyword, self.__root, value, self.__apply_permissions, self.__writable
                     )
             if not error:
                 super().set(value)
@@ -306,7 +306,7 @@ class OptionList(Option):
 
     def default_string(self) -> str:
         """Return the default list as a comma-separated string"""
-        return ", ".join(self.default())
+        return ", ".join(self.default)
 
     def __call__(self) -> List[str]:
         """get() replacement"""
@@ -364,7 +364,7 @@ class OptionPassword(Option):
 
     def get(self) -> Optional[str]:
         """Return decoded password"""
-        return decode_password(super().get(), self.ident())
+        return decode_password(super().get(), self.keyword)
 
     def get_stars(self) -> Optional[str]:
         """Return non-descript asterisk string"""
@@ -375,9 +375,9 @@ class OptionPassword(Option):
     def get_dict(self, safe: bool = False) -> Dict[str, str]:
         """Return value a dictionary"""
         if safe:
-            return {self.ident()[1]: self.get_stars()}
+            return {self.keyword: self.get_stars()}
         else:
-            return {self.ident()[1]: self.get()}
+            return {self.keyword: self.get()}
 
     def set(self, pw: str):
         """Set password, encode it"""
@@ -394,12 +394,12 @@ class ConfigServer:
 
     def __init__(self, name, values):
 
-        self.__name = name
+        self.__name = clean_section_name(name)
         name = "servers," + self.__name
 
         self.displayname = OptionStr(name, "displayname", add=False)
         self.host = OptionStr(name, "host", add=False)
-        self.port = OptionNumber(name, "port", 119, 0, 2 ** 16 - 1, add=False)
+        self.port = OptionNumber(name, "port", 119, 0, 2**16 - 1, add=False)
         self.timeout = OptionNumber(name, "timeout", 60, 20, 240, add=False)
         self.username = OptionStr(name, "username", add=False)
         self.password = OptionPassword(name, "password", add=False)
@@ -409,6 +409,7 @@ class ConfigServer:
         self.ssl_verify = OptionNumber(name, "ssl_verify", 2, add=False)
         self.ssl_ciphers = OptionStr(name, "ssl_ciphers", add=False)
         self.enable = OptionBool(name, "enable", True, add=False)
+        self.required = OptionBool(name, "required", False, add=False)
         self.optional = OptionBool(name, "optional", False, add=False)
         self.retention = OptionNumber(name, "retention", 0, add=False)
         self.expire_date = OptionStr(name, "expire_date", add=False)
@@ -442,6 +443,7 @@ class ConfigServer:
             "ssl_ciphers",
             "send_group",
             "enable",
+            "required",
             "optional",
             "retention",
             "expire_date",
@@ -476,6 +478,7 @@ class ConfigServer:
         output_dict["ssl_verify"] = self.ssl_verify()
         output_dict["ssl_ciphers"] = self.ssl_ciphers()
         output_dict["enable"] = self.enable()
+        output_dict["required"] = self.required()
         output_dict["optional"] = self.optional()
         output_dict["retention"] = self.retention()
         output_dict["expire_date"] = self.expire_date()
@@ -494,22 +497,19 @@ class ConfigServer:
         """Give server new display name"""
         self.displayname.set(name)
 
-    def ident(self) -> Tuple[str, str]:
-        return "servers", self.__name
-
 
 class ConfigCat:
     """Class defining a single category"""
 
     def __init__(self, name: str, values: Dict[str, Any]):
-        self.__name = name
-        name = "categories," + name
+        self.__name = clean_section_name(name)
+        name = "categories," + self.__name
 
         self.order = OptionNumber(name, "order", 0, 0, 100, add=False)
         self.pp = OptionStr(name, "pp", add=False)
         self.script = OptionStr(name, "script", "Default", add=False)
         self.dir = OptionDir(name, "dir", add=False, create=False)
-        self.newzbin = OptionList(name, "newzbin", add=False, validation=validate_single_tag)
+        self.newzbin = OptionList(name, "newzbin", add=False, validation=sabnzbd.cfg.validate_single_tag)
         self.priority = OptionNumber(name, "priority", DEFAULT_PRIORITY, add=False)
 
         self.set_dict(values)
@@ -605,8 +605,8 @@ class ConfigRSS:
     """Class defining a single Feed definition"""
 
     def __init__(self, name, values):
-        self.__name = name
-        name = "rss," + name
+        self.__name = clean_section_name(name)
+        name = "rss," + self.__name
 
         self.uri = OptionList(name, "uri", add=False)
         self.cat = OptionStr(name, "cat", add=False)
@@ -656,31 +656,42 @@ class ConfigRSS:
         self.__name = new_name
         add_to_database("rss", self.__name, self)
 
-    def ident(self) -> Tuple[str, str]:
-        return "rss", self.__name
+
+# Add typing to the options database-dict
+AllConfigTypes = Union[
+    Option,
+    OptionStr,
+    OptionPassword,
+    OptionNumber,
+    OptionBool,
+    OptionList,
+    OptionDir,
+    ConfigCat,
+    ConfigRSS,
+    ConfigServer,
+]
+CFG_DATABASE: Dict[str, Dict[str, AllConfigTypes]] = {}
 
 
 @synchronized(CONFIG_LOCK)
-def add_to_database(section, keyword, obj):
+def add_to_database(section: str, keyword: str, obj: AllConfigTypes):
     """add object as section/keyword to INI database"""
-    global database
-    if section not in database:
-        database[section] = {}
-    database[section][keyword] = obj
+    global CFG_DATABASE
+    if section not in CFG_DATABASE:
+        CFG_DATABASE[section] = {}
+    CFG_DATABASE[section][keyword] = obj
 
 
 @synchronized(CONFIG_LOCK)
 def delete_from_database(section, keyword):
     """Remove section/keyword from INI database"""
-    global database, CFG, modified
-    del database[section][keyword]
-    if section == "servers" and "[" in keyword:
-        keyword = keyword.replace("[", "{").replace("]", "}")
+    global CFG_DATABASE, CFG_OBJ, CFG_MODIFIED
+    del CFG_DATABASE[section][keyword]
     try:
-        del CFG[section][keyword]
+        del CFG_OBJ[section][keyword]
     except KeyError:
         pass
-    modified = True
+    CFG_MODIFIED = True
 
 
 def get_dconfig(section, keyword, nested=False):
@@ -689,13 +700,13 @@ def get_dconfig(section, keyword, nested=False):
     """
     data = {}
     if not section:
-        for section in database.keys():
+        for section in CFG_DATABASE.keys():
             res, conf = get_dconfig(section, None, True)
             data.update(conf)
 
     elif not keyword:
         try:
-            sect = database[section]
+            sect = CFG_DATABASE[section]
         except KeyError:
             return False, {}
         if section in ("servers", "categories", "rss"):
@@ -711,7 +722,7 @@ def get_dconfig(section, keyword, nested=False):
 
     else:
         try:
-            item = database[section][keyword]
+            item = CFG_DATABASE[section][keyword]
         except KeyError:
             return False, {}
         data = item.get_dict(safe=True)
@@ -724,10 +735,10 @@ def get_dconfig(section, keyword, nested=False):
     return True, data
 
 
-def get_config(section, keyword):
+def get_config(section: str, keyword: str) -> Optional[AllConfigTypes]:
     """Return a config object, based on 'section', 'keyword'"""
     try:
-        return database[section][keyword]
+        return CFG_DATABASE[section][keyword]
     except KeyError:
         logging.debug("Missing configuration item %s,%s", section, keyword)
         return None
@@ -736,7 +747,7 @@ def get_config(section, keyword):
 def set_config(kwargs):
     """Set a config item, using values in dictionary"""
     try:
-        item = database[kwargs.get("section")][kwargs.get("keyword")]
+        item = CFG_DATABASE[kwargs.get("section")][kwargs.get("keyword")]
     except KeyError:
         return False
     item.set_dict(kwargs)
@@ -746,7 +757,7 @@ def set_config(kwargs):
 def delete(section: str, keyword: str):
     """Delete specific config item"""
     try:
-        database[section][keyword].delete()
+        CFG_DATABASE[section][keyword].delete()
     except KeyError:
         return
 
@@ -769,7 +780,7 @@ def _read_config(path, try_backup=False):
     """Read the complete INI file and check its version number
     if OK, pass values to config-database
     """
-    global CFG, database, modified
+    global CFG_OBJ, CFG_DATABASE, CFG_MODIFIED
 
     if try_backup or not os.path.exists(path):
         # Not found, try backup
@@ -793,7 +804,7 @@ def _read_config(path, try_backup=False):
 
     try:
         # Let configobj open the file
-        CFG = configobj.ConfigObj(infile=path, default_encoding="utf-8", encoding="utf-8")
+        CFG_OBJ = configobj.ConfigObj(infile=path, default_encoding="utf-8", encoding="utf-8")
     except (IOError, configobj.ConfigObjError, UnicodeEncodeError) as strerror:
         if try_backup:
             # No luck!
@@ -803,86 +814,72 @@ def _read_config(path, try_backup=False):
             return _read_config(path, True)
 
     try:
-        version = sabnzbd.misc.int_conv(CFG["__version__"])
+        version = sabnzbd.misc.int_conv(CFG_OBJ["__version__"])
         if version > int(CONFIG_VERSION):
             return False, "Incorrect version number %s in %s" % (version, path)
     except (KeyError, ValueError):
         pass
 
-    CFG.filename = path
-    CFG.encoding = "utf-8"
-    CFG["__encoding__"] = "utf-8"
-    CFG["__version__"] = str(CONFIG_VERSION)
+    CFG_OBJ.filename = path
+    CFG_OBJ.encoding = "utf-8"
+    CFG_OBJ["__encoding__"] = "utf-8"
+    CFG_OBJ["__version__"] = str(CONFIG_VERSION)
 
     # Use CFG data to set values for all static options
-    for section in database:
+    for section in CFG_DATABASE:
         if section not in ("servers", "categories", "rss"):
-            for option in database[section]:
-                sec, kw = database[section][option].ident()
-                sec = sec[-1]
+            for option in CFG_DATABASE[section]:
+                config_option = CFG_DATABASE[section][option]
                 try:
-                    database[section][option].set(CFG[sec][kw])
+                    config_option.set(CFG_OBJ[config_option.section][config_option.keyword])
                 except KeyError:
                     pass
 
     # Define the special settings
-    if "categories" in CFG:
-        for cat in CFG["categories"]:
-            ConfigCat(cat, CFG["categories"][cat])
-    if "rss" in CFG:
-        for rss_feed in CFG["rss"]:
-            ConfigRSS(rss_feed, CFG["rss"][rss_feed])
-    if "servers" in CFG:
-        for server in CFG["servers"]:
-            ConfigServer(server.replace("{", "[").replace("}", "]"), CFG["servers"][server])
+    if "categories" in CFG_OBJ:
+        for cat in CFG_OBJ["categories"]:
+            ConfigCat(cat, CFG_OBJ["categories"][cat])
+    if "rss" in CFG_OBJ:
+        for rss_feed in CFG_OBJ["rss"]:
+            ConfigRSS(rss_feed, CFG_OBJ["rss"][rss_feed])
+    if "servers" in CFG_OBJ:
+        for server in CFG_OBJ["servers"]:
+            ConfigServer(server, CFG_OBJ["servers"][server])
 
-    modified = False
+    CFG_MODIFIED = False
     return True, ""
 
 
 @synchronized(SAVE_CONFIG_LOCK)
 def save_config(force=False):
     """Update Setup file with current option values"""
-    global CFG, database, modified
+    global CFG_OBJ, CFG_DATABASE, CFG_MODIFIED
 
-    if not (modified or force):
+    if not (CFG_MODIFIED or force):
         return True
 
     if sabnzbd.cfg.configlock():
         logging.warning(T("Configuration locked, cannot save settings"))
         return False
 
-    for section in database:
+    for section in CFG_DATABASE:
         if section in ("servers", "categories", "rss"):
-            try:
-                CFG[section]
-            except KeyError:
-                CFG[section] = {}
-            for subsec in database[section]:
-                if section == "servers":
-                    subsec_mod = subsec.replace("[", "{").replace("]", "}")
-                else:
-                    subsec_mod = subsec
-                try:
-                    CFG[section][subsec_mod]
-                except KeyError:
-                    CFG[section][subsec_mod] = {}
-                items = database[section][subsec].get_dict()
-                CFG[section][subsec_mod] = items
+            if section not in CFG_OBJ:
+                CFG_OBJ[section] = {}
+
+            for subsection in CFG_DATABASE[section]:
+                if subsection not in CFG_OBJ[section]:
+                    CFG_OBJ[section][subsection] = {}
+                CFG_OBJ[section][subsection] = CFG_DATABASE[section][subsection].get_dict()
         else:
-            for option in database[section]:
-                sec, kw = database[section][option].ident()
-                sec = sec[-1]
-                try:
-                    CFG[sec]
-                except KeyError:
-                    CFG[sec] = {}
-                value = database[section][option]()
-                # bool is a subclass of int, check first
-                CFG[sec][kw] = value
+            for option in CFG_DATABASE[section]:
+                config_option = CFG_DATABASE[section][option]
+                if config_option.section not in CFG_OBJ:
+                    CFG_OBJ[config_option.section] = {}
+                CFG_OBJ[config_option.section][config_option.keyword] = CFG_DATABASE[section][option]()
 
     res = False
-    filename = CFG.filename
+    filename = CFG_OBJ.filename
     bakname = filename + ".bak"
 
     # Check if file is writable
@@ -903,9 +900,9 @@ def save_config(force=False):
     # Write new config file
     try:
         logging.info("Writing settings to INI file %s", filename)
-        CFG.write()
+        CFG_OBJ.write()
         shutil.copymode(bakname, filename)
-        modified = False
+        CFG_MODIFIED = False
         res = True
     except:
         logging.error(T("Cannot write to INI file %s"), filename)
@@ -920,10 +917,67 @@ def save_config(force=False):
     return res
 
 
-def get_servers() -> Dict[str, ConfigServer]:
-    global database
+def create_config_backup():
+    """Put config data in a zip file"""
+    adminpath = sabnzbd.cfg.admin_dir.get_path()
+    logging.debug("Backing up %s + %s", adminpath, CFG_OBJ.filename)
+    with io.BytesIO() as zip_buffer:
+        with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_ref:
+            for filename in CONFIG_BACKUP_FILES:
+                full_path = os.path.join(adminpath, filename)
+                if os.path.isfile(full_path):
+                    with open(full_path, "rb") as data:
+                        zip_ref.writestr(filename, data.read())
+            with open(CFG_OBJ.filename, "rb") as data:
+                zip_ref.writestr(DEF_INI_FILE, data.read())
+        return zip_buffer.getvalue()
+
+
+def validate_config_backup(config_backup_data: bytes) -> bool:
+    """Check that the zip file contains a sabnzbd.ini"""
     try:
-        return database["servers"]
+        with io.BytesIO(config_backup_data) as backup_ref:
+            with zipfile.ZipFile(backup_ref, "r") as zip_ref:
+                # Will throw KeyError if not present
+                zip_ref.getinfo(DEF_INI_FILE)
+                return True
+    except:
+        return False
+
+
+def restore_config_backup(config_backup_data: bytes) -> bool:
+    """Restore configuration files from zip file"""
+    try:
+        with io.BytesIO(config_backup_data) as backup_ref:
+            with zipfile.ZipFile(backup_ref, "r") as zip_ref:
+                # Write config file first and read it
+                logging.debug("Writing backup of config-file to %s", CFG_OBJ.filename)
+                with open(CFG_OBJ.filename, "wb") as destination_ref:
+                    destination_ref.write(zip_ref.read(DEF_INI_FILE))
+                logging.debug("Loading settings from backup config-file")
+                read_config(CFG_OBJ.filename)
+
+                # Write the rest of the admin files that we want to recover
+                adminpath = sabnzbd.cfg.admin_dir.get_path()
+                for filename in CONFIG_BACKUP_FILES:
+                    try:
+                        zip_ref.getinfo(filename)
+                        destination_file = os.path.join(adminpath, filename)
+                        logging.debug("Writing backup of %s to %s", filename, destination_file)
+                        with open(destination_file, "wb") as destination_ref:
+                            destination_ref.write(zip_ref.read(filename))
+                    except KeyError:
+                        # File not in archive
+                        pass
+    except:
+        logging.warning(T("Could not restore backup"))
+        logging.info("Traceback: ", exc_info=True)
+
+
+def get_servers() -> Dict[str, ConfigServer]:
+    global CFG_DATABASE
+    try:
+        return CFG_DATABASE["servers"]
     except KeyError:
         return {}
 
@@ -932,10 +986,10 @@ def get_categories() -> Dict[str, ConfigCat]:
     """Return link to categories section.
     This section will always contain special category '*'
     """
-    global database
-    if "categories" not in database:
-        database["categories"] = {}
-    cats = database["categories"]
+    global CFG_DATABASE
+    if "categories" not in CFG_DATABASE:
+        CFG_DATABASE["categories"] = {}
+    cats = CFG_DATABASE["categories"]
 
     # Add Default categories
     if "*" not in cats:
@@ -980,11 +1034,11 @@ def get_ordered_categories() -> List[Dict]:
 
 
 def get_rss() -> Dict[str, ConfigRSS]:
-    global database
+    global CFG_DATABASE
     try:
         # We have to remove non-separator commas by detecting if they are valid URL's
-        for feed_key in database["rss"]:
-            feed = database["rss"][feed_key]
+        for feed_key in CFG_DATABASE["rss"]:
+            feed = CFG_DATABASE["rss"][feed_key]
             # Only modify if we have to, to prevent repeated config-saving
             have_new_uri = False
             # Create a new corrected list
@@ -1001,29 +1055,29 @@ def get_rss() -> Dict[str, ConfigRSS]:
             if have_new_uri:
                 feed.uri.set(new_feed_uris)
 
-        return database["rss"]
+        return CFG_DATABASE["rss"]
     except KeyError:
         return {}
 
 
 def get_filename():
-    global CFG
-    return CFG.filename
+    global CFG_OBJ
+    return CFG_OBJ.filename
+
+
+def clean_section_name(section: str) -> str:
+    """Make a section name suitable to be used in the INI,
+    since it can't have starting "[" or a trailing "]".
+    Unfortuantly, ConfigObj doesn't do this for us."""
+    new_section_name = section.strip("[]")
+    if not new_section_name:
+        raise ValueError("Invalid section name %s, nothing left after cleaning" % section)
+    return new_section_name
 
 
-##############################################################################
-# Default Validation handlers
-##############################################################################
 __PW_PREFIX = "!!!encoded!!!"
 
 
-class ErrorCatchingArgumentParser(argparse.ArgumentParser):
-    def error(self, status=0, message=None):
-        # Need to override so it doesn't raise SystemExit
-        if status:
-            raise ValueError
-
-
 def encode_password(pw):
     """Encode password in hexadecimal if needed"""
     enc = False
@@ -1039,7 +1093,7 @@ def encode_password(pw):
     return pw
 
 
-def decode_password(pw, name):
+def decode_password(pw: str, name: str) -> str:
     """Decode hexadecimal encoded password
     but only decode when prefixed
     """
@@ -1057,90 +1111,6 @@ def decode_password(pw, name):
         return pw
 
 
-def clean_nice_ionice_parameters(value):
-    """Verify that the passed parameters are not exploits"""
-    if value:
-        parser = ErrorCatchingArgumentParser()
-
-        # Nice parameters
-        parser.add_argument("-n", "--adjustment", type=int)
-
-        # Ionice parameters, not supporting -p
-        parser.add_argument("--classdata", type=int)
-        parser.add_argument("-c", "--class", type=int)
-        parser.add_argument("-t", "--ignore", action="store_true")
-
-        try:
-            parser.parse_args(value.split())
-        except ValueError:
-            # Also log at start-up if invalid parameter was set in the ini
-            msg = "%s: %s" % (T("Incorrect parameter"), value)
-            logging.error(msg)
-            return msg, None
-    return None, value
-
-
-def all_lowercase(value):
-    """Lowercase everything!"""
-    if isinstance(value, list):
-        # If list, for each item
-        return None, [item.lower() for item in value]
-    return None, value.lower()
-
-
-def validate_octal(value):
-    """Check if string is valid octal number"""
-    if not value:
-        return None, value
-    try:
-        int(value, 8)
-        return None, value
-    except:
-        return T("%s is not a correct octal value") % value, None
-
-
-def validate_no_unc(root, value, default):
-    """Check if path isn't a UNC path"""
-    # Only need to check the 'value' part
-    if value and not value.startswith(r"\\"):
-        return validate_notempty(root, value, default)
-    else:
-        return T('UNC path "%s" not allowed here') % value, None
-
-
-def validate_safedir(root, value, default):
-    """Allow only when queues are empty and no UNC"""
-    if not sabnzbd.__INITIALIZED__ or (sabnzbd.PostProcessor.empty() and sabnzbd.NzbQueue.is_empty()):
-        return validate_no_unc(root, value, default)
-    else:
-        return T("Error: Queue not empty, cannot change folder."), None
-
-
-def validate_notempty(root, value, default):
-    """If value is empty, return default"""
-    if value:
-        return None, value
-    else:
-        return None, default
-
-
-def validate_strip_right_slash(value):
-    """Strips the right slash"""
-    if value:
-        return None, value.rstrip("/")
-    return None, value
-
-
-def validate_single_tag(value: List[str]) -> Tuple[None, List[str]]:
-    """Don't split single indexer tags like "TV > HD"
-    into ['TV', '>', 'HD']
-    """
-    if len(value) == 3:
-        if value[1] == ">":
-            return None, [" ".join(value)]
-    return None, value
-
-
 def create_api_key():
     """Return a new randomized API_KEY"""
     return uuid.uuid4().hex
