@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,19 +24,18 @@ import select
 import logging
 from math import ceil
 from threading import Thread, RLock
-from nntplib import NNTPPermanentError
 import socket
 import random
 import sys
+import ssl
 from typing import List, Dict, Optional, Union
 
 import sabnzbd
 from sabnzbd.decorators import synchronized, NzbQueueLocker, DOWNLOADER_CV
-from sabnzbd.newswrapper import NewsWrapper
-import sabnzbd.notifier
+from sabnzbd.newswrapper import NewsWrapper, NNTPPermanentError
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import from_units, nntp_to_msg, int_conv, get_server_addrinfo
+from sabnzbd.misc import from_units, nntp_to_msg, get_server_addrinfo, helpful_warning
 from sabnzbd.utils.happyeyeballs import happyeyeballs
 
 
@@ -73,6 +72,8 @@ class Server:
         "ssl",
         "ssl_verify",
         "ssl_ciphers",
+        "ssl_context",
+        "required",
         "optional",
         "retention",
         "send_group",
@@ -103,12 +104,13 @@ class Server:
         timeout,
         threads,
         priority,
-        ssl,
+        use_ssl,
         ssl_verify,
         ssl_ciphers,
         send_group,
         username=None,
         password=None,
+        required=False,
         optional=False,
         retention=0,
     ):
@@ -122,9 +124,11 @@ class Server:
         self.timeout: int = timeout
         self.threads: int = threads
         self.priority: int = priority
-        self.ssl: bool = ssl
+        self.ssl: bool = use_ssl
         self.ssl_verify: int = ssl_verify
         self.ssl_ciphers: str = ssl_ciphers
+        self.ssl_context: Optional[ssl.SSLContext] = None
+        self.required: bool = required
         self.optional: bool = optional
         self.retention: int = retention
         self.send_group: bool = send_group
@@ -183,7 +187,7 @@ class Server:
                 logging.debug("%s: Connecting to address %s", self.host, ip)
             elif cfg.load_balancing() == 2:
                 # RFC6555 / Happy Eyeballs:
-                ip = happyeyeballs(self.host, port=self.port, ssl=self.ssl)
+                ip = happyeyeballs(self.host, port=self.port)
                 if ip:
                     logging.debug("%s: Connecting to address %s", self.host, ip)
                 else:
@@ -312,6 +316,7 @@ class Downloader(Thread):
             ssl_ciphers = srv.ssl_ciphers()
             username = srv.username()
             password = srv.password()
+            required = srv.required()
             optional = srv.optional()
             retention = int(srv.retention() * 24 * 3600)  # days ==> seconds
             send_group = srv.send_group()
@@ -344,6 +349,7 @@ class Downloader(Thread):
                     send_group,
                     username,
                     password,
+                    required,
                     optional,
                     retention,
                 )
@@ -363,8 +369,11 @@ class Downloader(Thread):
 
     @NzbQueueLocker
     def set_paused_state(self, state: bool):
-        """Set downloader to specified paused state"""
-        self.paused = state
+        """Set downloader to new paused state if it is changed"""
+        if self.paused != state:
+            if cfg.preserve_paused_state():
+                cfg.start_paused.set(state)
+            self.paused = state
 
     @NzbQueueLocker
     def resume(self):
@@ -372,6 +381,8 @@ class Downloader(Thread):
         if self.paused and sabnzbd.WEB_DIR:
             logging.info("Resuming")
             sabnzbd.notifier.send_notification("SABnzbd", T("Resuming"), "pause_resume")
+            if cfg.preserve_paused_state():
+                cfg.start_paused.set(False)
         self.paused = False
 
     @NzbQueueLocker
@@ -381,6 +392,8 @@ class Downloader(Thread):
             self.paused = True
             logging.info("Pausing")
             sabnzbd.notifier.send_notification("SABnzbd", T("Paused"), "pause_resume")
+            if cfg.preserve_paused_state():
+                cfg.start_paused.set(True)
             if self.is_paused():
                 sabnzbd.BPSMeter.reset()
             if cfg.autodisconnect():
@@ -413,7 +426,7 @@ class Downloader(Thread):
                 if mx:
                     self.bandwidth_limit = mx * self.bandwidth_perc / 100
                 else:
-                    logging.warning_helpful(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
+                    helpful_warning(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
             else:
                 self.bandwidth_limit = from_units(value)
                 if mx:
@@ -462,9 +475,6 @@ class Downloader(Thread):
                 return False
         return True
 
-    def nzo_servers(self, nzo):
-        return list(filter(nzo.server_in_try_list, self.servers))
-
     def maybe_block_server(self, server: Server):
         # Was it resolving problem?
         if server.info is False:
@@ -473,12 +483,16 @@ class Downloader(Thread):
             if server.errormsg != errormsg:
                 server.errormsg = errormsg
                 logging.warning(errormsg)
-                logging.warning(T("Server %s will be ignored for %s minutes"), server.host, _PENALTY_TIMEOUT)
+                if not server.required:
+                    logging.warning(T("Server %s will be ignored for %s minutes"), server.host, _PENALTY_TIMEOUT)
 
             # Not fully the same as the code below for optional servers
             server.bad_cons = 0
-            server.deactivate()
-            self.plan_server(server, _PENALTY_TIMEOUT)
+            if server.required:
+                sabnzbd.Scheduler.plan_required_server_resume()
+            else:
+                server.deactivate()
+                self.plan_server(server, _PENALTY_TIMEOUT)
 
         # Optional and active server had too many problems.
         # Disable it now and send a re-enable plan to the scheduler
@@ -508,33 +522,51 @@ class Downloader(Thread):
             if not article.search_new_server():
                 sabnzbd.NzbQueue.register_article(article, success=False)
                 article.nzf.nzo.increase_bad_articles_counter("missing_articles")
+                article.nzf.has_bad_articles = True
             return
 
         # Send to decoder-queue
         sabnzbd.Decoder.process(article, raw_data)
 
         # See if we need to delay because the queues are full
-        logged = False
-        while not self.shutdown and (sabnzbd.Decoder.queue_full() or sabnzbd.Assembler.queue_full()):
-            if not logged:
-                # Only log once, to not waste any CPU-cycles
+        logged_counter = 0
+        decoder_full = sabnzbd.Decoder.queue_full()
+        assembler_full = sabnzbd.Assembler.queue_full()
+        while not self.shutdown and (decoder_full or assembler_full):
+            # Only log/update once every second, to not waste any CPU-cycles
+            if not logged_counter % 10:
+                # Make sure the BPS-meter is updated
+                sabnzbd.BPSMeter.update()
+
+                # Update who is delaying us
+                sabnzbd.BPSMeter.delayed_decoder += int(decoder_full)
+                sabnzbd.BPSMeter.delayed_assembler += int(assembler_full)
                 logging.debug(
-                    "Delaying - Decoder queue: %s - Assembler queue: %s",
+                    "Delayed - %d seconds - Decoder queue: %d - Assembler queue: %d",
+                    logged_counter / 10,
                     sabnzbd.Decoder.decoder_queue.qsize(),
                     sabnzbd.Assembler.queue.qsize(),
                 )
-                logged = True
-            time.sleep(0.01)
+
+            # Wait and update the queue sizes
+            time.sleep(0.1)
+            logged_counter += 1
+            decoder_full = sabnzbd.Decoder.queue_full()
+            assembler_full = sabnzbd.Assembler.queue_full()
 
     def run(self):
         # First check IPv6 connectivity
-        sabnzbd.EXTERNAL_IPV6 = sabnzbd.test_ipv6()
+        sabnzbd.EXTERNAL_IPV6 = sabnzbd.misc.test_ipv6()
         logging.debug("External IPv6 test result: %s", sabnzbd.EXTERNAL_IPV6)
 
         # Then we check SSL certificate checking
-        sabnzbd.CERTIFICATE_VALIDATION = sabnzbd.test_cert_checking()
+        sabnzbd.CERTIFICATE_VALIDATION = sabnzbd.misc.test_cert_checking()
         logging.debug("SSL verification test: %s", sabnzbd.CERTIFICATE_VALIDATION)
 
+        # Warn if there are servers defined, but none are valid
+        if config.get_servers() and not self.servers:
+            logging.warning(T("There are no active servers!"))
+
         # Kick BPS-Meter to check quota
         BPSMeter = sabnzbd.BPSMeter
         BPSMeter.update()
@@ -772,11 +804,9 @@ class Downloader(Thread):
                             # Handle login problems
                             block = False
                             penalty = 0
-                            msg = error.response
-                            ecode = int_conv(msg[:3])
-                            display_msg = " [%s]" % msg
-                            logging.debug("Server login problem: %s, %s", ecode, msg)
-                            if ecode in (502, 400, 481, 482) and clues_too_many(msg):
+                            display_msg = " [%s]" % error.msg
+                            logging.debug("Server login problem: %s", error.msg)
+                            if error.code in (502, 400, 481, 482) and clues_too_many(error.msg):
                                 # Too many connections: remove this thread and reduce thread-setting for server
                                 # Plan to go back to the full number after a penalty timeout
                                 if server.active:
@@ -788,7 +818,7 @@ class Downloader(Thread):
                                     self.__reset_nw(nw, send_quit=True)
                                     self.plan_server(server, _PENALTY_TOOMANY)
                                     server.threads -= 1
-                            elif ecode in (502, 481, 482) and clues_too_many_ip(msg):
+                            elif error.code in (502, 481, 482) and clues_too_many_ip(error.msg):
                                 # Account sharing?
                                 if server.active:
                                     errormsg = T("Probable account sharing") + display_msg
@@ -798,7 +828,9 @@ class Downloader(Thread):
                                         logging.warning(T("Probable account sharing") + name)
                                 penalty = _PENALTY_SHARE
                                 block = True
-                            elif ecode in (452, 481, 482, 381) or (ecode == 502 and clues_login(msg)):
+                            elif error.code in (452, 481, 482, 381) or (
+                                error.code in (500, 502) and clues_login(error.msg)
+                            ):
                                 # Cannot login, block this server
                                 if server.active:
                                     errormsg = T("Failed login for server %s") % display_msg
@@ -807,19 +839,19 @@ class Downloader(Thread):
                                         logging.error(T("Failed login for server %s"), server.host)
                                 penalty = _PENALTY_PERM
                                 block = True
-                            elif ecode in (502, 482):
+                            elif error.code in (502, 482):
                                 # Cannot connect (other reasons), block this server
                                 if server.active:
-                                    errormsg = T("Cannot connect to server %s [%s]") % ("", display_msg)
+                                    errormsg = T("Cannot connect to server %s [%s]") % ("", error.msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, msg)
-                                if clues_pay(msg):
+                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
+                                if clues_pay(error.msg):
                                     penalty = _PENALTY_PERM
                                 else:
                                     penalty = _PENALTY_502
                                 block = True
-                            elif ecode == 400:
+                            elif error.code == 400:
                                 # Temp connection problem?
                                 if server.active:
                                     logging.debug("Unspecified error 400 from server %s", server.host)
@@ -831,16 +863,21 @@ class Downloader(Thread):
                                     errormsg = T("Cannot connect to server %s [%s]") % ("", display_msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, msg)
+                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
                                 penalty = _PENALTY_UNKNOWN
                                 block = True
                             if block or (penalty and server.optional):
+                                retry_article = False
                                 if server.active:
-                                    server.deactivate()
-                                    if penalty and (block or server.optional):
-                                        self.plan_server(server, penalty)
-                                # Note that the article is discard for this server
-                                self.__reset_nw(nw, retry_article=False, send_quit=True)
+                                    if server.required:
+                                        sabnzbd.Scheduler.plan_required_server_resume()
+                                        retry_article = True
+                                    else:
+                                        server.deactivate()
+                                        if penalty and (block or server.optional):
+                                            self.plan_server(server, penalty)
+                                # Note that the article is discard for this server if the server is not required
+                                self.__reset_nw(nw, retry_article=retry_article, send_quit=True)
                             continue
                         except:
                             logging.error(
@@ -929,7 +966,7 @@ class Downloader(Thread):
         # Make sure it is not in the readable sockets
         self.remove_socket(nw)
 
-        if nw.article:
+        if nw.article and not nw.article.nzf.nzo.removed_from_queue:
             # Only some errors should count towards the total tries for each server
             if count_article_try:
                 nw.article.tries += 1
@@ -1127,3 +1164,17 @@ def check_server_quota():
                 logging.warning(T("Server %s has used the specified quota"), server.displayname())
                 server.quota.set("")
                 config.save_config()
+
+
+def pause_all():
+    """Pause all activities than cause disk access"""
+    sabnzbd.PAUSED_ALL = True
+    sabnzbd.Downloader.pause()
+    logging.debug("PAUSED_ALL active")
+
+
+def unpause_all():
+    """Resume all activities"""
+    sabnzbd.PAUSED_ALL = False
+    sabnzbd.Downloader.resume()
+    logging.debug("PAUSED_ALL inactive")
