@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,20 +24,20 @@ import time
 from datetime import datetime
 import cherrypy
 import logging
-import urllib.request, urllib.parse, urllib.error
+import urllib.parse
 import re
 import hashlib
 import socket
 import ssl
 import functools
+import copy
 from random import randint
 from xml.sax.saxutils import escape
 from Cheetah.Template import Template
-from typing import Optional, Callable, Union
+from typing import Optional, Callable, Union, Any, Dict
 from guessit.api import properties as guessit_properties
 
 import sabnzbd
-import sabnzbd.rss
 from sabnzbd.misc import (
     to_units,
     from_units,
@@ -51,6 +51,8 @@ from sabnzbd.misc import (
     is_lan_addr,
     is_loopback_addr,
     ip_in_subnet,
+    helpful_warning,
+    recursive_html_escape,
 )
 from sabnzbd.filesystem import (
     real_path,
@@ -66,8 +68,9 @@ import sabnzbd.cfg as cfg
 import sabnzbd.notifier as notifier
 import sabnzbd.newsunpack
 from sabnzbd.utils.servertests import test_nntp_server_dict
+from sabnzbd.utils.getperformance import getcpu
 import sabnzbd.utils.ssdp
-from sabnzbd.constants import DEF_STDCONFIG, DEFAULT_PRIORITY, CHEETAH_DIRECTIVES, EXCLUDED_GUESSIT_PROPERTIES
+from sabnzbd.constants import DEF_STD_CONFIG, DEFAULT_PRIORITY, CHEETAH_DIRECTIVES, EXCLUDED_GUESSIT_PROPERTIES
 from sabnzbd.lang import list_languages
 from sabnzbd.api import (
     list_scripts,
@@ -142,12 +145,16 @@ def secured_expose(
         # Check if config is locked
         if check_configlock and cfg.configlock():
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED_CONFIG_LOCK
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED_CONFIG_LOCK
+            return
 
         # Check if external access and if it's allowed
         if not check_access(access_type=access_type, warn_user=True):
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED
+            return
 
         # Verify login status, only for non-key pages
         if check_for_login and not check_api_key and not check_login():
@@ -156,14 +163,18 @@ def secured_expose(
         # Verify host used for the visit
         if not check_hostname():
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED_HOSTNAME
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED_HOSTNAME
+            return
 
         # Some pages need correct API key
         if check_api_key:
             msg = check_apikey(kwargs)
             if msg:
                 cherrypy.response.status = 403
-                return msg
+                if cfg.api_warnings():
+                    return msg
+                return
 
         # All good, cool!
         return wrap_func(*args, **kwargs)
@@ -364,6 +375,15 @@ def check_apikey(kwargs):
     return None
 
 
+def template_filtered_response(file: str, search_list: Dict[str, Any]):
+    """Wrapper for Cheetah response"""
+    # We need a copy, because otherwise source-dicts might be modified
+    search_list_copy = copy.deepcopy(search_list)
+    # 'filters' is excluded because the RSS-filters are listed twice
+    recursive_html_escape(search_list_copy, exclude_items=("webdir", "filters"))
+    return Template(file=file, searchList=[search_list_copy], compilerSettings=CHEETAH_DIRECTIVES).respond()
+
+
 def log_warning_and_ip(txt):
     """Include the IP and the Proxy-IP for warnings"""
     if cfg.api_warnings():
@@ -415,6 +435,9 @@ class MainPage:
             info["have_rss_defined"] = bool(config.get_rss())
             info["have_watched_dir"] = bool(cfg.dirscan_dir())
 
+            info["cpumodel"] = getcpu()
+            info["cpusimd"] = sabnzbd.decoder.SABYENC_SIMD
+
             # Have logout only with HTML and if inet=5, only when we are external
             info["have_logout"] = (
                 cfg.username()
@@ -428,10 +451,7 @@ class MainPage:
             bytespersec_list = sabnzbd.BPSMeter.get_bps_list()
             info["bytespersec_list"] = ",".join([str(bps) for bps in bytespersec_list])
 
-            template = Template(
-                file=os.path.join(sabnzbd.WEB_DIR, "main.tmpl"), searchList=[info], compilerSettings=CHEETAH_DIRECTIVES
-            )
-            return template.respond()
+            return template_filtered_response(file=os.path.join(sabnzbd.WEB_DIR, "main.tmpl"), search_list=info)
         else:
             # Redirect to the setup wizard
             raise cherrypy.HTTPRedirect("%s/wizard/" % cfg.url_base())
@@ -494,10 +514,8 @@ class Wizard:
 
         info = build_header(sabnzbd.WIZARD_DIR)
         info["languages"] = list_languages()
-        template = Template(
-            file=os.path.join(sabnzbd.WIZARD_DIR, "index.html"), searchList=[info], compilerSettings=CHEETAH_DIRECTIVES
-        )
-        return template.respond()
+
+        return template_filtered_response(file=os.path.join(sabnzbd.WIZARD_DIR, "index.html"), search_list=info)
 
     @secured_expose(check_configlock=True)
     def one(self, **kwargs):
@@ -505,9 +523,6 @@ class Wizard:
         if kwargs.get("lang"):
             cfg.language.set(kwargs.get("lang"))
 
-        # Always setup Glitter
-        change_web_dir("Glitter - Auto")
-
         info = build_header(sabnzbd.WIZARD_DIR)
         info["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
@@ -540,10 +555,7 @@ class Wizard:
                 info["ssl_verify"] = s.ssl_verify()
                 if s.enable():
                     break
-        template = Template(
-            file=os.path.join(sabnzbd.WIZARD_DIR, "one.html"), searchList=[info], compilerSettings=CHEETAH_DIRECTIVES
-        )
-        return template.respond()
+        return template_filtered_response(file=os.path.join(sabnzbd.WIZARD_DIR, "one.html"), search_list=info)
 
     @secured_expose(check_configlock=True)
     def two(self, **kwargs):
@@ -562,10 +574,7 @@ class Wizard:
         info["download_dir"] = cfg.download_dir.get_clipped_path()
         info["complete_dir"] = cfg.complete_dir.get_clipped_path()
 
-        template = Template(
-            file=os.path.join(sabnzbd.WIZARD_DIR, "two.html"), searchList=[info], compilerSettings=CHEETAH_DIRECTIVES
-        )
-        return template.respond()
+        return template_filtered_response(file=os.path.join(sabnzbd.WIZARD_DIR, "two.html"), search_list=info)
 
 
 def get_access_info():
@@ -573,14 +582,16 @@ def get_access_info():
     # Access_url is used to provide the user a link to SABnzbd depending on the host
     cherryhost = cfg.cherryhost()
     host = socket.gethostname().lower()
+    logging.info("hostname is", host)
     socks = [host]
 
+    try:
+        addresses = socket.getaddrinfo(host, None)
+    except:
+        addresses = []
+
     if cherryhost == "0.0.0.0":
         # Grab a list of all ips for the hostname
-        try:
-            addresses = socket.getaddrinfo(host, None)
-        except:
-            addresses = []
         for addr in addresses:
             address = addr[4][0]
             # Filter out ipv6 addresses (should not be allowed)
@@ -589,7 +600,6 @@ def get_access_info():
         socks.insert(0, "localhost")
     elif cherryhost == "::":
         # Grab a list of all ips for the hostname
-        addresses = socket.getaddrinfo(host, None)
         for addr in addresses:
             address = addr[4][0]
             # Only ipv6 addresses will work
@@ -650,12 +660,10 @@ class LoginPage:
             logging.warning(T("Unsuccessful login attempt from %s"), cherrypy.request.remote_label)
 
         # Show login
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "login", "main.tmpl"),
-            searchList=[info],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=info,
         )
-        return template.respond()
 
 
 ##############################################################################
@@ -681,19 +689,23 @@ class ConfigPage:
         conf["build"] = sabnzbd.__baseline__[:7]
 
         conf["have_unzip"] = bool(sabnzbd.newsunpack.ZIP_COMMAND)
-        conf["have_7zip"] = bool(sabnzbd.newsunpack.SEVEN_COMMAND)
+        conf["have_7zip"] = bool(sabnzbd.newsunpack.SEVENZIP_COMMAND)
         conf["have_sabyenc"] = sabnzbd.decoder.SABYENC_ENABLED
         conf["have_mt_par2"] = sabnzbd.newsunpack.PAR2_MT
 
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
         conf["ssl_version"] = ssl.OPENSSL_VERSION
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
+
+    @secured_expose(check_configlock=True, check_api_key=True)
+    def backup(self, **kwargs):
+        cherrypy.response.headers["Content-Type"] = "application/zip"
+        cherrypy.response.headers["Content-Disposition"] = 'attachment; filename="sabnzbd-config.zip"'
+        return config.create_config_backup()
 
 
 ##############################################################################
@@ -727,12 +739,10 @@ class ConfigFolders:
         for kw in LIST_DIRPAGE + LIST_BOOL_DIRPAGE:
             conf[kw] = config.get_config("misc", kw)()
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_folders.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveDirectories(self, **kwargs):
@@ -747,7 +757,7 @@ class ConfigFolders:
                     # return sabnzbd.api.report('json', error=msg)
                     return badParameterResponse(msg, kwargs.get("ajax"))
 
-        if not sabnzbd.check_incomplete_vs_complete():
+        if not sabnzbd.filesystem.check_incomplete_vs_complete():
             return badParameterResponse(
                 T("The Completed Download Folder cannot be the same or a subfolder of the Temporary Download Folder"),
                 kwargs.get("ajax"),
@@ -801,26 +811,8 @@ SWITCH_LIST = (
     "unwanted_extensions",
     "action_on_unwanted_extensions",
     "unwanted_extensions_mode",
+    "cleanup_list",
     "sanitize_safe",
-    "rating_enable",
-    "rating_api_key",
-    "rating_filter_enable",
-    "rating_filter_abort_audio",
-    "rating_filter_abort_video",
-    "rating_filter_abort_encrypted",
-    "rating_filter_abort_encrypted_confirm",
-    "rating_filter_abort_spam",
-    "rating_filter_abort_spam_confirm",
-    "rating_filter_abort_downvoted",
-    "rating_filter_abort_keywords",
-    "rating_filter_pause_audio",
-    "rating_filter_pause_video",
-    "rating_filter_pause_encrypted",
-    "rating_filter_pause_encrypted_confirm",
-    "rating_filter_pause_spam",
-    "rating_filter_pause_spam_confirm",
-    "rating_filter_pause_downvoted",
-    "rating_filter_pause_keywords",
 )
 
 
@@ -835,37 +827,26 @@ class ConfigSwitches:
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
         conf["have_nice"] = bool(sabnzbd.newsunpack.NICE_COMMAND)
         conf["have_ionice"] = bool(sabnzbd.newsunpack.IONICE_COMMAND)
-        conf["cleanup_list"] = cfg.cleanup_list.get_string()
 
         for kw in SWITCH_LIST:
             conf[kw] = config.get_config("misc", kw)()
+        conf["cleanup_list"] = cfg.cleanup_list.get_string()
         conf["unwanted_extensions"] = cfg.unwanted_extensions.get_string()
 
         conf["scripts"] = list_scripts() or ["None"]
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_switches.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSwitches(self, **kwargs):
         for kw in SWITCH_LIST:
-            item = config.get_config("misc", kw)
-            value = kwargs.get(kw)
-            if kw == "unwanted_extensions" and value:
-                value = value.lower().replace(".", "")
-            msg = item.set(value)
+            msg = config.get_config("misc", kw).set(kwargs.get(kw))
             if msg:
                 return badParameterResponse(msg, kwargs.get("ajax"))
 
-        cleanup_list = kwargs.get("cleanup_list")
-        if cleanup_list and sabnzbd.WIN32:
-            cleanup_list = cleanup_list.lower()
-        cfg.cleanup_list.set(cleanup_list)
-
         config.save_config()
         if kwargs.get("ajax"):
             return sabnzbd.api.report()
@@ -876,6 +857,7 @@ class ConfigSwitches:
 ##############################################################################
 SPECIAL_BOOL_LIST = (
     "start_paused",
+    "preserve_paused_state",
     "no_penalties",
     "fast_fail",
     "overwrite_files",
@@ -883,7 +865,7 @@ SPECIAL_BOOL_LIST = (
     "process_unpacked_par2",
     "queue_complete_pers",
     "api_warnings",
-    "helpfull_warnings",
+    "helpful_warnings",
     "ampm",
     "enable_unrar",
     "enable_unzip",
@@ -908,7 +890,7 @@ SPECIAL_BOOL_LIST = (
     "disable_api_key",
     "api_logging",
     "x_frame_options",
-    "require_modern_tls",
+    "allow_old_ssl_tls",
 )
 SPECIAL_VALUE_LIST = (
     "downloader_sleep_time",
@@ -922,11 +904,10 @@ SPECIAL_VALUE_LIST = (
     "max_foldername_length",
     "show_sysload",
     "url_base",
-    "num_decoders",
+    "num_simd_decoders",
     "direct_unpack_threads",
     "ipv6_servers",
     "selftest_host",
-    "rating_host",
     "ssdp_broadcast_interval",
 )
 SPECIAL_LIST_LIST = (
@@ -934,6 +915,7 @@ SPECIAL_LIST_LIST = (
     "quick_check_ext_ignore",
     "host_whitelist",
     "local_ranges",
+    "ext_rename_ignore",
 )
 
 
@@ -945,10 +927,10 @@ class ConfigSpecial:
     def index(self, **kwargs):
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
         conf["switches"] = [
-            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default()) for kw in SPECIAL_BOOL_LIST
+            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default) for kw in SPECIAL_BOOL_LIST
         ]
         conf["entries"] = [
-            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default()) for kw in SPECIAL_VALUE_LIST
+            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default) for kw in SPECIAL_VALUE_LIST
         ]
         conf["entries"].extend(
             [
@@ -957,12 +939,10 @@ class ConfigSpecial:
             ]
         )
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_special.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSpecial(self, **kwargs):
@@ -982,7 +962,6 @@ GENERAL_LIST = (
     "host",
     "port",
     "username",
-    "refresh_rate",
     "language",
     "cache_limit",
     "inet_exposure",
@@ -992,6 +971,7 @@ GENERAL_LIST = (
     "https_key",
     "https_chain",
     "enable_https_verification",
+    "socks5_proxy_url",
     "auto_browser",
     "check_new_rel",
 )
@@ -1006,13 +986,12 @@ class ConfigGeneral:
 
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
 
-        conf["configfn"] = config.get_filename()
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
         web_list = []
         for interface_dir in globber_full(sabnzbd.DIR_INTERFACES):
             # Ignore the config
-            if not interface_dir.endswith(DEF_STDCONFIG):
+            if not interface_dir.endswith(DEF_STD_CONFIG):
                 # Check the available templates
                 for colorscheme in globber(
                     os.path.join(interface_dir, "templates", "static", "stylesheets", "colorschemes")
@@ -1034,12 +1013,10 @@ class ConfigGeneral:
         conf["nzb_key"] = cfg.nzb_key()
         conf["caller_url"] = cherrypy.request.base + cfg.url_base()
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_general.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveGeneral(self, **kwargs):
@@ -1065,7 +1042,7 @@ class ConfigGeneral:
             cfg.bandwidth_perc.set(bandwidth_perc)
         bandwidth_perc = cfg.bandwidth_perc()
         if bandwidth_perc and not bandwidth_max:
-            logging.warning_helpful(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
+            helpful_warning(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
 
         config.save_config()
 
@@ -1076,6 +1053,22 @@ class ConfigGeneral:
         else:
             raise Raiser(self.__root)
 
+    @secured_expose(check_api_key=True, check_configlock=True)
+    def uploadConfig(self, **kwargs):
+        """Restore a config backup"""
+        config_backup_file = kwargs.get("config_backup_file")
+
+        # Only accept the backup file if it can be opened as a zip archive and only contains a config file
+        try:
+            config_backup_data = config_backup_file.file.read()
+            config_backup_file.file.close()
+            if config.validate_config_backup(config_backup_data):
+                sabnzbd.RESTORE_DATA = config_backup_data
+                return sabnzbd.api.report(data={"success": True, "restart_req": True})
+        except:
+            pass
+        return sabnzbd.api.report(error=T("Invalid backup archive"))
+
 
 def change_web_dir(web_dir):
     web_dir, web_color = web_dir.split(" - ")
@@ -1124,12 +1117,10 @@ class ConfigServer:
         conf["cats"] = list_cats(default=True)
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_server.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def addServer(self, **kwargs):
@@ -1234,7 +1225,7 @@ def handle_server(kwargs, root=None, new_svr=False):
     if new_svr:
         server = unique_svr_name(server)
 
-    for kw in ("ssl", "send_group", "enable", "optional"):
+    for kw in ("ssl", "send_group", "enable", "required", "optional"):
         if kw not in kwargs.keys():
             kwargs[kw] = None
     if svr and not new_svr:
@@ -1334,12 +1325,10 @@ class ConfigRss:
             unum += 1
         conf["feed"] = txt + str(unum)
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_rss.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def save_rss_rate(self, **kwargs):
@@ -1463,7 +1452,7 @@ class ConfigRss:
 
         if filt:
             feed_cfg.filters.update(
-                int(kwargs.get("index", 0)), (cat, pp, script, kwargs.get("filter_type"), filt, prio, enabled)
+                int(kwargs.get("index", 0)), [cat, pp, script, kwargs.get("filter_type"), filt, prio, enabled]
             )
 
             # Move filter if requested
@@ -1562,7 +1551,7 @@ class ConfigRss:
             prio = att.get("prio")
 
             if url:
-                sabnzbd.add_url(url, pp, script, cat, prio, nzbname)
+                sabnzbd.urlgrabber.add_url(url, pp, script, cat, prio, nzbname)
             # Need to pass the title instead
             sabnzbd.RSSReader.flag_downloaded(feed, url)
         raise rssRaiser(self.__root, kwargs)
@@ -1709,12 +1698,10 @@ class ConfigScheduling:
         conf["actions_lng"] = actions_lng
         conf["categories"] = categories
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_scheduling.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def addSchedule(self, **kwargs):
@@ -1807,11 +1794,6 @@ class ConfigCats:
         categories = config.get_ordered_categories()
         conf["have_cats"] = len(categories) > 1
 
-        slotinfo = []
-        for cat in categories:
-            cat["newzbin"] = cat["newzbin"].replace('"', "&quot;")
-            slotinfo.append(cat)
-
         # Add empty line
         empty = {
             "name": "",
@@ -1822,15 +1804,13 @@ class ConfigCats:
             "newzbin": "",
             "priority": DEFAULT_PRIORITY,
         }
-        slotinfo.insert(1, empty)
-        conf["slotinfo"] = slotinfo
+        categories.insert(1, empty)
+        conf["slotinfo"] = categories
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_cat.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def delete(self, **kwargs):
@@ -1842,10 +1822,10 @@ class ConfigCats:
     @secured_expose(check_api_key=True, check_configlock=True)
     def save(self, **kwargs):
         name = kwargs.get("name", "*")
+        newname = kwargs.get("newname", "")
         if name == "*":
             newname = name
-        else:
-            newname = re.sub('"', "", kwargs.get("newname", ""))
+
         if newname:
             # Check if this cat-dir is not sub-folder of incomplete
             if same_file(cfg.download_dir.get_path(), real_path(cfg.complete_dir.get_path(), kwargs["dir"])):
@@ -1868,7 +1848,6 @@ SORT_LIST = (
     "enable_movie_sorting",
     "movie_sort_string",
     "movie_sort_extra",
-    "movie_extra_folder",
     "enable_date_sorting",
     "date_sort_string",
     "movie_categories",
@@ -1892,12 +1871,10 @@ class ConfigSorting:
             prop for prop in guessit_properties().keys() if prop not in EXCLUDED_GUESSIT_PROPERTIES
         )
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_sorting.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSorting(self, **kwargs):
@@ -2178,7 +2155,7 @@ class ConfigNotify:
         conf["notify_types"] = sabnzbd.notifier.NOTIFICATION
         conf["categories"] = list_cats(False)
         conf["have_ntfosd"] = sabnzbd.notifier.have_ntfosd()
-        conf["have_ncenter"] = sabnzbd.DARWIN and sabnzbd.FOUNDATION
+        conf["have_ncenter"] = sabnzbd.MACOS and sabnzbd.FOUNDATION
         conf["scripts"] = list_scripts(default=False, none=True)
 
         for section in NOTIFY_OPTIONS:
@@ -2186,12 +2163,10 @@ class ConfigNotify:
                 # Use get_string to make sure lists are displayed correctly
                 conf[option] = config.get_config(section, option).get_string()
 
-        template = Template(
+        return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_notify.tmpl"),
-            searchList=[conf],
-            compilerSettings=CHEETAH_DIRECTIVES,
+            search_list=conf,
         )
-        return template.respond()
 
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveNotify(self, **kwargs):
