@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@ sabnzbd.newswrapper
 import errno
 import socket
 from threading import Thread
-from nntplib import NNTPPermanentError
 import time
 import logging
 import ssl
@@ -38,6 +37,16 @@ from sabnzbd.misc import nntp_to_msg, is_ipv4_addr, is_ipv6_addr, get_server_add
 socket.setdefaulttimeout(DEF_TIMEOUT)
 
 
+class NNTPPermanentError(Exception):
+    def __init__(self, msg: str, code: int):
+        super().__init__()
+        self.msg = msg
+        self.code = code
+
+    def __str__(self) -> str:
+        return self.msg
+
+
 class NewsWrapper:
     # Pre-define attributes to save memory
     __slots__ = (
@@ -114,8 +123,8 @@ class NewsWrapper:
             self.pass_sent = False
             self.pass_ok = False
 
-        if code in (400, 502):
-            raise NNTPPermanentError(nntp_to_msg(self.data))
+        if code in (400, 500, 502):
+            raise NNTPPermanentError(nntp_to_msg(self.data), code)
         elif not self.user_sent:
             command = utob("authinfo user %s\r\n" % self.server.username)
             self.nntp.sock.sendall(command)
@@ -139,7 +148,7 @@ class NewsWrapper:
         elif self.user_ok and not self.pass_ok:
             if code != 281:
                 # Assume that login failed (code 481 or other)
-                raise NNTPPermanentError(nntp_to_msg(self.data))
+                raise NNTPPermanentError(nntp_to_msg(self.data), code)
             else:
                 self.connected = True
 
@@ -268,7 +277,7 @@ class NNTP:
         if not self.nw.server.info:
             raise socket.error(errno.EADDRNOTAVAIL, "Address not available - Check for internet or DNS problems")
 
-        af, socktype, proto, canonname, sa = self.nw.server.info[0]
+        af, socktype, proto, _, _ = self.nw.server.info[0]
 
         # there will be a connect to host (or self.host, so let's force set 'af' to the correct value
         if is_ipv4_addr(self.host):
@@ -276,38 +285,42 @@ class NNTP:
         if is_ipv6_addr(self.host):
             af = socket.AF_INET6
 
-        # Secured or unsecured?
-        if not self.nw.server.ssl:
-            # Basic connection
-            self.sock = socket.socket(af, socktype, proto)
-        else:
-            # Use context or just wrapper
-            if sabnzbd.CERTIFICATE_VALIDATION:
-                # Setup the SSL socket
-                ctx = ssl.create_default_context()
+        # Create SSL-context if it is needed and not created yet
+        if self.nw.server.ssl and not self.nw.server.ssl_context:
+            # Setup the SSL socket
+            self.nw.server.ssl_context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
 
-                if sabnzbd.cfg.require_modern_tls():
-                    # We want a modern TLS (1.2 or higher), so we disallow older protocol versions (<= TLS 1.1)
-                    ctx.options |= ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 | ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1
-
-                # Only verify hostname when we're strict
-                if self.nw.server.ssl_verify < 2:
-                    ctx.check_hostname = False
+            # Only verify hostname when we're strict
+            if self.nw.server.ssl_verify < 2:
+                self.nw.server.ssl_context.check_hostname = False
                 # Certificates optional
                 if self.nw.server.ssl_verify == 0:
-                    ctx.verify_mode = ssl.CERT_NONE
+                    self.nw.server.ssl_context.verify_mode = ssl.CERT_NONE
 
-                # Did the user set a custom cipher-string?
-                if self.nw.server.ssl_ciphers:
-                    # At their own risk, socket will error out in case it was invalid
-                    ctx.set_ciphers(self.nw.server.ssl_ciphers)
-
-                self.sock = ctx.wrap_socket(socket.socket(af, socktype, proto), server_hostname=self.nw.server.host)
+            # Did the user set a custom cipher-string?
+            if self.nw.server.ssl_ciphers:
+                # At their own risk, socket will error out in case it was invalid
+                self.nw.server.ssl_context.set_ciphers(self.nw.server.ssl_ciphers)
+                # Python does not allow setting ciphers on TLSv1.3, so have to force TLSv1.2 as the maximum
+                self.nw.server.ssl_context.maximum_version = ssl.TLSVersion.TLSv1_2
             else:
-                # Use a regular wrapper, no certificate validation
-                self.sock = ssl.wrap_socket(socket.socket(af, socktype, proto))
+                # Support at least TLSv1.2+ ciphers, as some essential ones are removed by default in Python 3.10
+                self.nw.server.ssl_context.set_ciphers("HIGH")
 
-        # Store fileno of the socket
+            if sabnzbd.cfg.allow_old_ssl_tls():
+                # Allow anything that the system has
+                self.nw.server.ssl_context.minimum_version = ssl.TLSVersion.MINIMUM_SUPPORTED
+            else:
+                # We want a modern TLS (1.2 or higher), so we disallow older protocol versions (<= TLS 1.1)
+                self.nw.server.ssl_context.minimum_version = ssl.TLSVersion.TLSv1_2
+
+            # Disable any verification if the setup is bad
+            if not sabnzbd.CERTIFICATE_VALIDATION:
+                self.nw.server.ssl_context.check_hostname = False
+                self.nw.server.ssl_context.verify_mode = ssl.CERT_NONE
+
+        # Create socket and store fileno of the socket
+        self.sock = socket.socket(af, socktype, proto)
         self.fileno: int = self.sock.fileno()
 
         # Open the connection in a separate thread due to avoid blocking
@@ -326,10 +339,11 @@ class NNTP:
 
             # Connect
             self.sock.connect((self.host, self.nw.server.port))
-            self.sock.setblocking(self.nw.blocking)
 
-            # Log SSL/TLS info
+            # Secured or unsecured?
             if self.nw.server.ssl:
+                # Wrap socket and log SSL/TLS diagnostic info
+                self.sock = self.nw.server.ssl_context.wrap_socket(self.sock, server_hostname=self.nw.server.host)
                 logging.info(
                     "%s@%s: Connected using %s (%s)",
                     self.nw.thrdnum,
@@ -339,6 +353,9 @@ class NNTP:
                 )
                 self.nw.server.ssl_info = "%s (%s)" % (self.sock.version(), self.sock.cipher()[0])
 
+            # Set blocking mode
+            self.sock.setblocking(self.nw.blocking)
+
             # Now it's safe to add the socket to the list of active sockets
             # Skip this step during server test
             if not self.nw.blocking:
